@@ -140,17 +140,20 @@ gdm_server_stop (GdmDisplay *disp)
     disp->servstat = SERVER_DEAD;
 
     if (disp->servpid != 0) {
-	    pid_t servpid = disp->servpid;
+	    pid_t servpid;
+
+	    gdm_debug ("gdm_server_stop: Killing server pid %d",
+		       (int)disp->servpid);
 
 	    /* avoid SIGCHLD race */
+	    gdm_sigchld_block_push ();
+
+	    servpid = disp->servpid;
 	    disp->servpid = 0;
 
-	    gdm_debug ("gdm_server_stop: Killing server pid %d", (int)servpid);
-
-	    gdm_sigchld_block_push ();
 	    if (servpid > 0 &&
 		kill (servpid, SIGTERM) == 0)
-		    waitpid (servpid, 0, 0);
+		    ve_waitpid_no_signal (servpid, 0, 0);
 	    gdm_sigchld_block_pop ();
 
 
@@ -399,7 +402,7 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
      * sockets etc. If the old X server isn't completely dead, the new
      * one will fail and we'll hang here forever */
 
-    /* Only do alarm if esrver will be run as root */
+    /* Only do alarm if server will be run as root */
     if (d->server_uid == 0) {
 	    alarm (SERVER_WAIT_ALARM);
     }
@@ -409,11 +412,22 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
     /* fork X server process */
     gdm_server_spawn (d);
 
+    /* we can now use d->handled since that's set up above */
+
     /* Wait for X server to send ready signal */
     if (d->servstat == SERVER_STARTED) {
-	    if (d->server_uid != 0) {
+	    if (d->server_uid != 0 && ! d->handled) {
+		    alarm (0);
+		    /* FIXME: If not handled, we just don't know, so
+		     * just wait a few seconds and hope things just work,
+		     * fortunately there is no such case yet and probably
+		     * never will, but just for code anality's sake */
+		    sleep (5);
+	    } else if (d->server_uid != 0) {
 		    int i;
 
+		    alarm (0);
+
 		    /* if we're running the server as a non-root, we can't
 		     * use USR1 of course, so try openning the display 
 		     * as a test, but the */
@@ -498,12 +512,14 @@ gdm_server_start (GdmDisplay *disp, gboolean treat_as_flexi,
 
     /* bad things are happening */
     if (d->servpid > 0) {
-	    pid_t pid = d->servpid;
-	    d->servpid = 0;
+	    pid_t pid;
+
 	    gdm_sigchld_block_push ();
+	    pid = d->servpid;
+	    d->servpid = 0;
 	    if (pid > 0 &&
 		kill (pid, SIGTERM) == 0)
-		    waitpid (pid, NULL, 0);
+		    ve_waitpid_no_signal (pid, NULL, 0);
 	    gdm_sigchld_block_pop ();
     }
 
@@ -613,11 +629,78 @@ rotate_logs (const char *dname)
 	g_free (fname);
 }
 
+char **
+gdm_server_resolve_command_line (GdmDisplay *disp,
+				 gboolean resolve_handled)
+{
+	char *bin;
+	char **argv;
+	int len;
+	int i;
+
+	bin = ve_first_word (disp->command);
+	if (bin == NULL) {
+		gdm_error (_("Invalid server command '%s'"), disp->command);
+		argv = ve_split (GdmStandardXServer);
+	} else if (bin[0] != '/') {
+		GdmXServer *svr = gdm_find_x_server (bin);
+		if (svr == NULL) {
+			gdm_error (_("Server name '%s' not found, "
+				     "using standard server"), bin);
+			argv = ve_split (GdmStandardXServer);
+		} else {
+			char **svr_command =
+				ve_split (ve_sure_string (svr->command));
+			argv = ve_split (disp->command);
+			if (argv[0] == NULL || argv[1] == NULL) {
+				g_strfreev (argv);
+				argv = svr_command;
+			} else {
+				char **old_argv = argv;
+				argv = ve_vector_merge (svr_command,
+							&old_argv[1]);
+				g_strfreev (svr_command);
+				g_strfreev (old_argv);
+			} 
+
+			if (resolve_handled)
+				/* Setup the handled function */
+				disp->handled = svr->handled;
+		}
+	} else {
+		argv = ve_split (disp->command);
+	}
+
+	for (len = 0; argv != NULL && argv[len] != NULL; len++)
+		;
+
+	argv = g_renew (char *, argv, len + 4);
+	for (i = len - 1; i >= 1; i--) {
+		argv[i+1] = argv[i];
+	}
+	/* server number is the FIRST argument, before any others */
+	argv[1] = g_strdup (disp->name);
+	if (disp->authfile != NULL) {
+		argv[len+1] = g_strdup ("-auth");
+		argv[len+2] = g_strdup (disp->authfile);
+		argv[len+3] = NULL;
+	} else {
+		argv[len+1] = NULL;
+	}
+
+	g_free (bin);
+
+	return argv;
+}
+
 /**
  * gdm_server_spawn:
  * @disp: Pointer to a GdmDisplay structure
  *
  * forks an actual X server process
+ *
+ * Note that we can only use d->handled once we call this function
+ * since otherwise the server might not yet be looked up yet.
  */
 
 static void 
@@ -625,12 +708,9 @@ gdm_server_spawn (GdmDisplay *d)
 {
     struct sigaction ign_signal, dfl_signal;
     sigset_t mask;
-    gchar *srvcmd = NULL;
     gchar **argv = NULL;
     int logfd;
-    int len, i;
-    const char *command;
-    char *bin;
+    char *command;
     pid_t pid;
 
     if (d == NULL ||
@@ -647,9 +727,15 @@ gdm_server_spawn (GdmDisplay *d)
 	    pid_t pid = d->servpid;
 	    d->servpid = 0;
 	    if (kill (pid, SIGTERM) == 0)
-		    waitpid (pid, NULL, 0);
+		    ve_waitpid_no_signal (pid, NULL, 0);
     }
 
+    /* Figure out the server command */
+    argv = gdm_server_resolve_command_line (d,
+					    TRUE /* resolve handled */);
+
+    command = g_strjoinv (" ", argv);
+
     /* Fork into two processes. Parent remains the gdm process. Child
      * becomes the X server. */
 
@@ -670,8 +756,7 @@ gdm_server_spawn (GdmDisplay *d)
 	closelog ();
 
 	/* close things */
-	for (i = 0; i < sysconf (_SC_OPEN_MAX); i++)
-		close(i);
+	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
 
 	/* No error checking here - if it's messed the best response
          * is to ignore & try to continue */
@@ -741,65 +826,14 @@ gdm_server_spawn (GdmDisplay *d)
 			gnome_unsetenv ("XAUTHORITY");
 	}
 
-	bin = ve_first_word (d->command);
-	if (bin == NULL) {
-		gdm_error (_("Invalid server command '%s'"), d->command);
-		argv = ve_split (GdmStandardXServer);
-		command = GdmStandardXServer;
-	} else if (bin[0] != '/') {
-		GdmXServer *svr = gdm_find_x_server (bin);
-		if (svr == NULL) {
-			gdm_error (_("Server name '%s' not found, "
-				     "using standard server"), bin);
-			argv = ve_split (GdmStandardXServer);
-			command = GdmStandardXServer;
-		} else {
-			char **svr_command =
-				ve_split (ve_sure_string (svr->command));
-			argv = ve_split (d->command);
-			if (argv[0] == NULL || argv[1] == NULL) {
-				g_strfreev (argv);
-				argv = svr_command;
-			} else {
-				char **old_argv = argv;
-				argv = ve_vector_merge (svr_command,
-							&old_argv[1]);
-				g_strfreev (svr_command);
-				g_strfreev (old_argv);
-			}
-			/* this is only for information only,
-			 * so doesn't include whole command line */
-			command = svr->command;
-		}
-	} else {
-		command = d->command;
-		argv = ve_split (d->command);
-	}
-	g_free (bin);
-
-	for (len = 0; argv != NULL && argv[len] != NULL; len++)
-		;
-
-	if (len == 0) {
+	if (argv[0] == NULL) {
 		gdm_error (_("%s: Empty server command for display %s"),
 			   "gdm_server_spawn",
 			   d->name);
 		_exit (SERVER_ABORT);
 	}
 
-	argv = g_renew (char *, argv, len + 4);
-	for (i = len - 1; i >= 1; i--) {
-		argv[i+1] = argv[i];
-	}
-	/* server number is the FIRST argument, before any others */
-	argv[1] = d->name;
-	argv[len+1] = "-auth";
-	argv[len+2] = d->authfile;
-	argv[len+3] = NULL;
-
-	srvcmd = g_strjoinv (" ", argv);
-	gdm_debug ("gdm_server_spawn: '%s'", srvcmd);
-	g_free (srvcmd);
+	gdm_debug ("gdm_server_spawn: '%s'", command);
 	
 	setpgid (0, 0);
 
@@ -857,6 +891,8 @@ gdm_server_spawn (GdmDisplay *d)
 	_exit (SERVER_ABORT);
 	
     case -1:
+	g_strfreev (argv);
+	g_free (command);
 	gdm_error (_("gdm_server_spawn: Can't fork Xserver process!"));
 	d->servpid = 0;
 	d->servstat = SERVER_DEAD;
@@ -864,6 +900,8 @@ gdm_server_spawn (GdmDisplay *d)
 	break;
 	
     default:
+	g_strfreev (argv);
+	g_free (command);
 	gdm_debug ("gdm_server_spawn: Forked server on pid %d", (int)pid);
 	break;
     }
@@ -980,8 +1018,9 @@ gdm_server_alloc (gint id, const gchar *command)
     gchar hostname[1024];
     GdmDisplay *d;
     
+    hostname[1023] = '\0';
     if (gethostname (hostname, 1023) == -1)
-	return NULL;
+	    strcmp (hostname, "localhost.localdomain");
 
     d = g_new0 (GdmDisplay, 1);
 
@@ -1011,10 +1050,14 @@ gdm_server_alloc (gint id, const gchar *command)
     d->screenx = 0; /* xinerama offset */
     d->screeny = 0;
 
+    d->handled = TRUE;
+
 #ifdef __linux__
     d->vt = -1;
 #endif
 
+    d->x_servers_order = -1;
+
     d->last_start_time = 0;
     d->retry_count = 0;
     d->disabled = FALSE;
