@@ -102,7 +102,10 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 
   if (item->item != NULL)
     return;
-  
+
+  if ( ! greeter_item_is_visible (item))
+    return;
+
   g_assert (item->parent->group_item);
   
   if (item->fixed_children != NULL ||
@@ -189,7 +192,7 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
   case GREETER_ITEM_TYPE_ENTRY:
     entry = gtk_entry_new ();
     gtk_entry_set_has_frame (GTK_ENTRY (entry), FALSE);
-    if (greeter_use_circles_in_entry)
+    if (GdmUseCirclesInEntry)
       gtk_entry_set_invisible_char (GTK_ENTRY (entry), 0x25cf);
     
     item->item = gnome_canvas_item_new (group,
@@ -205,9 +208,10 @@ greeter_item_create_canvas_item (GreeterItemInfo *item)
 
   if (item->item_type == GREETER_ITEM_TYPE_RECT ||
       item->item_type == GREETER_ITEM_TYPE_SVG ||
-      item->item_type == GREETER_ITEM_TYPE_PIXMAP)
-    gtk_signal_connect (GTK_OBJECT (item->item), "event",
-			(GtkSignalFunc) greeter_item_event_handler,
-			item);
+      item->item_type == GREETER_ITEM_TYPE_PIXMAP ||
+      item->item_type == GREETER_ITEM_TYPE_LABEL)
+    g_signal_connect (G_OBJECT (item->item), "event",
+		      G_CALLBACK (greeter_item_event_handler),
+		      item);
 }
 
